{ lib, config, ... }:
let fs = import ../../../lib/modules/fileSystems/fs.nix;
in {
  imports = [ ../../users/manuel/config.nix ];

  config = {
    valhalla = {
      boot.label = "Arch";

      fileSystems = {
        diskSetup.devices = {
          OS = {
            path = "/dev/sda";

            partitions = {
              Boot = {
                index = 1;
                type = "uefi";
                size = "+1G";
                format = fs.fat32;
                mountPoint = config.valhalla.boot.efiMountPoint;
              };

              Swap = {
                index = 2;
                type = "swap";
              };

              OS = {
                index = 3;
                label = lib.mkDefault config.valhalla.boot.label;
                type = "linux";
              };
            };
          };
        };

        btrfs = {
          volumes = {
            OS = {
              mountPoint = "/";
              devices = [ "/dev/sda3" "/dev/sdb" ];
              metadataProfile = "raid1";
              dataProfile = "single";
            };
          };
        };
      };

      hostname = "nuth.ch";
      timeZone = "Europe/Zurich";
      keyMap = "de_CH-latin1";
      keyboardLayout = "ch";

      i18n = {
        localeSettings =
          let defaultLocale = "en_US.UTF-8";
          in {
            LANG = "de_CH.UTF-8";
            LANGUAGE = defaultLocale;
            LC_MESSAGE = defaultLocale;
          };
      };

      software = {
        essential = true;
        server = true;
      };

      programs.docker.enable = true;

      linux.programs = {
        grub.enable = true;

        systemd-networkd = {
          enable = true;

          networks =
            let device = "enp0s31f6";
            in {
              ${device} = {
                Match = {
                  Name = device;
                };

                Network = {
                  Address = "2a01:4f8:10b:2644::2/64";
                  Gateway = [
                    "94.130.48.193"
                    "fe80::1"
                  ];

                  DNS = [
                    "5.9.164.112"
                    "1.1.1.1"
                  ];
                };

                Address = {
                  Address = "94.130.48.251";
                  Peer = "94.130.48.193/32";
                };
              };
            };
        };

        docker = {
          services = {
            anki-sync.enable = true;
            drone.enable = true;
            forgejo.enable = true;
            jellyfin.enable = true;
            minecraft.enable = true;
            nextcloud.enable = true;
            ryot.enable = true;
            teamspeak.enable = true;
            terraria.enable = true;
            trackmania.enable = true;
            vaultwarden.enable = true;
            wekan.enable = true;
            woodpecker.enable = true;
          };
        };
      };
    };
  };
}
